/********************************************************

		kaboom.h

   structures, constants and functions for fiber database
   and collision detection.

********************************************************/

#define MAXFIBER 288		/*  Number of fibers in system  */

#define MAXRED  96
#define MAXBLUE 96

#define NUM_FOPS 12

/* These two values must be computed by hand!
   The second number, PARKEDATRISK is the number
   of parked fibers to each side of an extended 
   fiber that must be checked for interaction.
   The second value must be determined empirically!
   if you increase the max angle, you must determine
   the second value by hand and re-set it!  Failure
   to do so may destroy hardware!
*/

/* #define STOWOFFSET   0.00168      angular offset for stow positions */
#define STOWOFFSET   0.00       /* angular offset for stow positions */


#define MAXANGLE    	 0.1745    /* 10 degrees */
#define ANGLE_SOFTNESS   0.0300    /* 1.0 degrees */
#define PARKEDATRISK 3		/* for 10 degrees */


#define BUTTONHEIGHT  9500	/* encoder steps old value= 19000 */
#define GRABHEIGHT    7300	/* encoder steps old value= 14600 */
#define BUTTONDIA    1400	/* encoder steps */
#define TUBEDIA      700	/* encoder steps */

#define BUTTON_TUBE_MARGIN    350	/* encoder steps */
#define BUTTON_BUTTON_MARGIN  150	/* encoder steps */

#define TUBE_ZONE     ((TUBEDIA / 2 ) + BUTTON_TUBE_MARGIN)

#define BUTTONSPACE  ( (BUTTONDIA / 2) + ( TUBEDIA / 2 ) + BUTTON_TUBE_MARGIN )
#define REQPREC	      100	/* encoder steps */

#define MAXEXTEND    ( PIVOT + ( BUTTONDIA / 4 ) ) /*  for normal buttons. */

#define CRITICALANGLE 2.356	/* 3PI/4 */
#define MAXFORPRECALC 15000     /* appx 1 1/2 inches */

#define DEFLECTION_RATIO 	0.475
/* Do not change either of these next two sets of numbers without modifying
   both. */

#define NUM_TUBE_SEGS 4	/* number of segments to represent a tube
				   with straight line segments. */

#define SEGPT0   0.0	/* fractions of tube length for endpoints of segs. */
#define SEGPT1   0.28868
#define SEGPT2   0.57735
#define SEGPT3   0.78867
#define SEGPT4   1.0


#define VOIDBUTTON -1
#define FREE    1
#define NOFREE  0

#define INITPARKED 0
#define INITSTOWED 1
#define DIVERGE  0
#define CONVERGE 1
#define PARALLELCCW 2
#define PARALLELCW  3

#define STATIC       0
#define MOVE_PENDING 1
#define IN_TRANSIT   2
#define AT_NEW_LOC   3
#define LOST         4

#define NO_TROUBLE 0
#define CANT_GRAB  1
#define CANT_DROP  2

#define NO_RESTRICT       0
#define RESTRICT_TO_PLATE 1

#define EXTENDED 0
#define PARKED   1
#define STOWED   2
#define TWEAKED  3

#define OVER_EXTEND 1
#define ON_STEP     2

#define BUTTON_BUTTON  1
#define BUTTON_TUBE    2
#define TUBE_TUBE      3
#define OCCULTED       4

#define NOHELP 1

#define IGNORANCE 1;
#define ENLIGHTENMENT 0;

#define UPDATE_FILE 1
#define DONT_FILE   0

#define FORCE_MOVE -1
#define DEFAULT_DEAD_ZONE 5
#define SKY_DEAD_ZONE 180	/* appx 3 arcsec */

#ifndef TRUE

#define TRUE	1
#define FALSE	0

#endif


/* structures */


struct eccentric_pair{
	double   mag;
	double angle;					/*  radians  */
	};


struct flex_geometry{

   /* The following must be re-calculated when the button is moved. */
   /* They are retained for each fiber for use in collision avoidance */

   double d1;		/*  extension of button along radial line */
   double d2;		/*  distance from origin to button  */
   double d3;		/*  portion of radial line not in d1 */
   double d4;		/*  distance from button to emergence point */
   double theta;	/*  angle at origin betw radial and "0" line */
   double phi;		/*  angle at origin between radial and d2 */
   double psi;		/*  angle at emergence betw. radial & d4 */
   double chi;		/*  angle of tube at button wrt radial */
   double shadow;       /*  shadow angle of button wrt origin */
   double defl;		/*  deflection of button from radial line */
   double c;		/*  magic constant for tube flexure */
   double buttonLocalSlope;	/*  slope of tube at button */
   double buttonCartSlope;	/*  slope of tube at button cartesian*/
   struct aPoint button;	/*  coordinates of button */
   struct aPoint pivot;   	/*  coordinates of emergence */
   struct aLine *twoPtAppx;  	/*  linear approximation of tube thru pivot */
   struct aLine *onePtAppx;  	/*  linear approximation of tube thru button */
   struct aLine *segments[NUM_TUBE_SEGS];  /* line segments for tube appx. */
};
	

struct fiber_dbase{

   int fiber_id;			/* equiv. to the array index */

   int slitPosition;			/* self explanatory */

   char cable;				/* ditto */

   void *assignedObject;		/* pointer to info about object */

   struct anOccult *occultedBy;		/* who is blocking us */
   struct anOccult *pendOccultBy;	/* new stuff after successful move */

   struct anOccult *weOccult;		/* who we are blocking */
   struct anOccult *pendWeOccult;	/* new stuff after successful move */


   struct plate_coords    CurPos;		/* current position */
   struct plate_coords    PriorPos;		/* prior position --used for error 
						   recovery purposes at boot */
   struct plate_coords    ParkPos;
   struct plate_coords    StowPos;
   struct plate_coords    PivotPos;
   struct eccentric_pair  Eccentricity;
   struct flex_geometry   *CurGeometry; /* only ptr changes when updating */
   struct flex_geometry   *TrialGeometry; /* only ptr changes when updating */

   double MaxAngle;		/*  radians  */
   long	MaxExtend;		/*  encoder steps  */
   long	ButtonHeight;		/*  encoder steps  */
   long	GrabHeight;		/*  encoder steps  */
   long	ButtonDia;		/*  encoder steps  */
   long	TubeDia;		/*  encoder steps  */
	
   int  lastMoveStatus; /* tells us what happened last time fiber was moved */
   int 	parkedStatus;   /* is the fiber PARKED or EXTENDED */
   int 	futureParkedStatus;	/* used by transition program */
   int	virtual;	/* boolean --fiber doesn't physically exist */
   int	broken;		/* boolean */
   int	locked;		/* boolean */
   int  tweaked;	/* indicates that an assigned button has been tweaked */
   int  updated;	/* used to tell if a fiber was moved by a script */
   int  outOfDate;	/* used by restoreRecent() in archive.c */

};

typedef struct fiber_dbase fibDB;


struct snapShot{  /* used to recover from error during button move */

   struct fiber_dbase *fiber;
   int state;
   int id;	/* afterthought so that we can safely flush/restore to file */
		/* Warning:  may not always be valid !! */
		/* set at beginning of move and never cleared!! */
		/* Use pointer to determine if structure is valid */

   };


extern struct fiber_dbase FiBase[];	/*  master database */
extern struct snapShot FiberInTransit;  /*  button that is being moved */
extern struct snapShot FiberInTrouble;  /*  button that is having problems */

extern int softAngleFlag;


extern void initDbase();
extern void initRadiusList(void);

extern int DestOK( fibDB fibers[], fibDB *fib );
extern int thisIsOK( fibDB fibers[], fibDB *fib );
extern int coordStatus( fibDB *fib, int restrictFlag );
extern int pairHits( fibDB *fib, fibDB *targetFib );

extern int overButton( fibDB *fiber1, point *thePoint );

extern int PlateZ( fibDB *fib, point *targetCoord, int *step, int boundryFlag );

extern struct flex_geometry *allocGeometry( void );
extern void freeGeometry(struct flex_geometry *pGeo);

extern int  prepNewLocation( fibDB *fib, point *target );
extern void recordNewLocation( fibDB *fib, int archiveFlag );
extern void restoreOldLocation();
extern void setLocation( fibDB *fib, struct plate_coords *location, int archiveFlag );
extern void setDbase( int flag );

extern double pendingDelta( fibDB *pFib );


